# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import utils
from kmip.core.messages import payloads


class TestObtainLeaseRequestPayload(testtools.TestCase):
    """
    Test suite for the ObtainLease request payload.
    """

    def setUp(self):
        super(TestObtainLeaseRequestPayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 9.5.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - f4152f17-9312-431a-b3fb-4fe86a86a7a1

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x66\x34\x31\x35\x32\x66\x31\x37\x2D\x39\x33\x31\x32\x2D\x34\x33'
            b'\x31\x61\x2D\x62\x33\x66\x62\x2D\x34\x66\x65\x38\x36\x61\x38\x36'
            b'\x61\x37\x61\x31\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestObtainLeaseRequestPayload, self).tearDown()

    def test_init(self):
        """
        Test that an ObtainLease request payload can be constructed with no
        arguments.
        """
        payload = payloads.ObtainLeaseRequestPayload()

        self.assertEqual(None, payload.unique_identifier)

    def test_init_with_args(self):
        """
        Test that an ObtainLease request payload can be constructed with valid
        values.
        """
        payload = payloads.ObtainLeaseRequestPayload(
            unique_identifier='00000000-1111-2222-3333-444444444444'
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of an ObtainLease request payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.ObtainLeaseRequestPayload,
            **kwargs
        )

        payload = payloads.ObtainLeaseRequestPayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that an ObtainLease request payload can be read from a data
        stream.
        """
        payload = payloads.ObtainLeaseRequestPayload()

        self.assertEqual(None, payload.unique_identifier)

        payload.read(self.full_encoding)

        self.assertEqual(
            'f4152f17-9312-431a-b3fb-4fe86a86a7a1',
            payload.unique_identifier
        )

    def test_read_empty(self):
        """
        Test that an ObtainLease request payload can be read from an empty
        data stream.
        """
        payload = payloads.ObtainLeaseRequestPayload()

        self.assertEqual(None, payload.unique_identifier)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)

    def test_write(self):
        """
        Test that an ObtainLease request payload can be written to a data
        stream.
        """
        payload = payloads.ObtainLeaseRequestPayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty ObtainLease request payload can be written to a
        data stream.
        """
        payload = payloads.ObtainLeaseRequestPayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        ObtainLease request payloads with the same data.
        """
        a = payloads.ObtainLeaseRequestPayload()
        b = payloads.ObtainLeaseRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.ObtainLeaseRequestPayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )
        b = payloads.ObtainLeaseRequestPayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease request payloads with different unique identifiers.
        """
        a = payloads.ObtainLeaseRequestPayload(
            unique_identifier='a'
        )
        b = payloads.ObtainLeaseRequestPayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease request payloads with different types.
        """
        a = payloads.ObtainLeaseRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        ObtainLease request payloads with the same data.
        """
        a = payloads.ObtainLeaseRequestPayload()
        b = payloads.ObtainLeaseRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.ObtainLeaseRequestPayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )
        b = payloads.ObtainLeaseRequestPayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease request payloads with different unique identifiers.
        """
        a = payloads.ObtainLeaseRequestPayload(
            unique_identifier='a'
        )
        b = payloads.ObtainLeaseRequestPayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease request payloads with different types.
        """
        a = payloads.ObtainLeaseRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to an ObtainLease request payload.
        """
        payload = payloads.ObtainLeaseRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )
        expected = (
            "ObtainLeaseRequestPayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038')"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to an ObtainLease request payload
        """
        payload = payloads.ObtainLeaseRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038'
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038'
        })
        observed = str(payload)

        self.assertEqual(expected, observed)


class TestObtainLeaseResponsePayload(testtools.TestCase):
    """
    Test suite for the ObtainLease response payload.
    """

    def setUp(self):
        super(TestObtainLeaseResponsePayload, self).setUp()

        # Encoding obtained from the KMIP 1.1 testing document, Section 9.5.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - f4152f17-9312-431a-b3fb-4fe86a86a7a1
        #     Lease Time - 0
        #     Last Change Date - 0x4F9A5564 (Fri Apr 27 10:14:28 CEST 2012)

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x50'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x66\x34\x31\x35\x32\x66\x31\x37\x2D\x39\x33\x31\x32\x2D\x34\x33'
            b'\x31\x61\x2D\x62\x33\x66\x62\x2D\x34\x66\x65\x38\x36\x61\x38\x36'
            b'\x61\x37\x61\x31\x00\x00\x00\x00'
            b'\x42\x00\x49\x0A\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x42\x00\x48\x09\x00\x00\x00\x08\x00\x00\x00\x00\x4F\x9A\x55\x64'
        )

        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - f4152f17-9312-431a-b3fb-4fe86a86a7a1
        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x30'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x66\x34\x31\x35\x32\x66\x31\x37\x2D\x39\x33\x31\x32\x2D\x34\x33'
            b'\x31\x61\x2D\x62\x33\x66\x62\x2D\x34\x66\x65\x38\x36\x61\x38\x36'
            b'\x61\x37\x61\x31\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestObtainLeaseResponsePayload, self).tearDown()

    def test_init(self):
        """
        Test that an ObtainLease response payload can be constructed with no
        arguments.
        """
        payload = payloads.ObtainLeaseResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

    def test_init_with_args(self):
        """
        Test that an ObtainLease response payload can be constructed with valid
        values.
        """
        payload = payloads.ObtainLeaseResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            lease_time=1000000000,
            last_change_date=1512400153
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(1000000000, payload.lease_time)
        self.assertEqual(1512400153, payload.last_change_date)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of an ObtainLease response payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.ObtainLeaseResponsePayload,
            **kwargs
        )

        payload = payloads.ObtainLeaseResponsePayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_lease_time(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the lease time of an ObtainLease response payload.
        """
        kwargs = {'lease_time': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            payloads.ObtainLeaseResponsePayload,
            **kwargs
        )

        payload = payloads.ObtainLeaseResponsePayload()
        args = (payload, 'lease_time', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            setattr,
            *args
        )

    def test_invalid_last_change_date(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the last change date of an ObtainLease response payload.
        """
        kwargs = {'last_change_date': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Last change date must be an integer.",
            payloads.ObtainLeaseResponsePayload,
            **kwargs
        )

        payload = payloads.ObtainLeaseResponsePayload()
        args = (payload, 'last_change_date', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Last change date must be an integer.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that an ObtainLease response payload can be read from a data
        stream.
        """
        payload = payloads.ObtainLeaseResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

        payload.read(self.full_encoding)

        self.assertEqual(
            'f4152f17-9312-431a-b3fb-4fe86a86a7a1',
            payload.unique_identifier
        )
        self.assertEqual(0, payload.lease_time)
        self.assertEqual(1335514468, payload.last_change_date)

    def test_read_partial(self):
        """
        Test that an ObtainLease response payload can be read from a partial
        data stream.
        """
        payload = payloads.ObtainLeaseResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

        payload.read(self.partial_encoding)

        self.assertEqual(
            'f4152f17-9312-431a-b3fb-4fe86a86a7a1',
            payload.unique_identifier
        )
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

    def test_read_empty(self):
        """
        Test that an ObtainLease response payload can be read from an empty
        data stream.
        """
        payload = payloads.ObtainLeaseResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.lease_time)
        self.assertEqual(None, payload.last_change_date)

    def test_write(self):
        """
        Test that an ObtainLease response payload can be written to a data
        stream.
        """
        payload = payloads.ObtainLeaseResponsePayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1',
            lease_time=0,
            last_change_date=1335514468
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial ObtainLease response payload can be written to a
        data stream.
        """
        payload = payloads.ObtainLeaseResponsePayload(
            unique_identifier='f4152f17-9312-431a-b3fb-4fe86a86a7a1'
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty ObtainLease response payload can be written to a
        data stream.
        """
        payload = payloads.ObtainLeaseResponsePayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        ObtainLease response payloads with the same data.
        """
        a = payloads.ObtainLeaseResponsePayload()
        b = payloads.ObtainLeaseResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882848,
            last_change_date=1512410153
        )
        b = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882848,
            last_change_date=1512410153
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease response payloads with different unique identifiers.
        """
        a = payloads.ObtainLeaseResponsePayload(
            unique_identifier='a'
        )
        b = payloads.ObtainLeaseResponsePayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_lease_time(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease response payloads with different lease times.
        """
        a = payloads.ObtainLeaseResponsePayload(
            lease_time=0
        )
        b = payloads.ObtainLeaseResponsePayload(
            lease_time=1511882848
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_last_change_date(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease response payloads with different last change dates.
        """
        a = payloads.ObtainLeaseResponsePayload(
            last_change_date=0
        )
        b = payloads.ObtainLeaseResponsePayload(
            last_change_date=1511882848
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        ObtainLease response payloads with different types.
        """
        a = payloads.ObtainLeaseResponsePayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        ObtainLease response payloads with the same data.
        """
        a = payloads.ObtainLeaseResponsePayload()
        b = payloads.ObtainLeaseResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882848,
            last_change_date=0
        )
        b = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882848,
            last_change_date=0
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease response payloads with different unique identifiers.
        """
        a = payloads.ObtainLeaseResponsePayload(
            unique_identifier='a'
        )
        b = payloads.ObtainLeaseResponsePayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_lease_time(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease response payloads with different lease times.
        """
        a = payloads.ObtainLeaseResponsePayload(
            lease_time=0
        )
        b = payloads.ObtainLeaseResponsePayload(
            lease_time=1511882848
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_last_change_time(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease response payloads with different last change time.
        """
        a = payloads.ObtainLeaseResponsePayload(
            lease_time=0
        )
        b = payloads.ObtainLeaseResponsePayload(
            lease_time=1511882848
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        ObtainLease response payloads with different types.
        """
        a = payloads.ObtainLeaseResponsePayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to an ObtainLease response payload.
        """
        payload = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882898,
            last_change_date=1512410153
        )
        expected = (
            "ObtainLeaseResponsePayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "lease_time=1511882898, "
            "last_change_date=1512410153)"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to an ObtainLease response payload.
        """
        payload = payloads.ObtainLeaseResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            lease_time=1511882898,
            last_change_date=1512410153
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'lease_time': 1511882898,
            'last_change_date': 1512410153
        })
        observed = str(payload)

        self.assertEqual(expected, observed)



# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from kmip.core import enums
from kmip.core import exceptions
from kmip.core import objects
from kmip.core import primitives
from kmip.core import utils
from kmip.core.messages.payloads import base


class ModifyAttributeRequestPayload(base.RequestPayload):
    """
    A request payload for the ModifyAttribute operation.

    Attributes:
        unique_identifier: The unique ID of the object on which attribute
            deletion should be performed.
        attribute: The attribute value to be modified by index. Used in KMIP
            1.0 - 1.4.
        current_attribute: The current attribute to be modified on the
            specified object. Used in KMIP 2.0+.
        new_attribute: The new attribute value to set on the specified object.
            Used in KMIP 2.0+.
    """

    def __init__(self,
                 unique_identifier=None,
                 attribute=None,
                 current_attribute=None,
                 new_attribute=None):
        """
        Construct a ModifyAttribute request payload.

        Args:
            unique_identifier (string): The unique ID of the object on which
                attribute modification should be performed. Optional, defaults
                to None.
            attribute (struct): An Attribute object containing the attribute
                name and index identifying the existing attribute, and the new
                attribute value to replace the original attribute. Optional,
                defaults to None. Used in KMIP 1.0 - 1.4. Required for
                read/write.
            current_attribute (struct): A CurrentAttribute object containing
                the original attribute value on the specified object. Optional,
                defaults to None. Used in KMIP 2.0+.
            new_attribute (struct): A NewAttribute object containing the new
                attribute value to set on the specified object. Optional,
                defaults to None. Used in KMIP 2.0+. Required for read/write.
        """
        super(ModifyAttributeRequestPayload, self).__init__()

        self._unique_identifier = None
        self._attribute = None
        self._current_attribute = None
        self._new_attribute = None

        self.unique_identifier = unique_identifier
        self.attribute = attribute
        self.current_attribute = current_attribute
        self.new_attribute = new_attribute

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("The unique identifier must be a string.")

    @property
    def attribute(self):
        if self._attribute:
            return self._attribute
        return None

    @attribute.setter
    def attribute(self, value):
        if value is None:
            self._attribute = None
        elif isinstance(value, objects.Attribute):
            self._attribute = value
        else:
            raise TypeError("The attribute must be an Attribute object.")

    @property
    def current_attribute(self):
        if self._current_attribute:
            return self._current_attribute
        return None

    @current_attribute.setter
    def current_attribute(self, value):
        if value is None:
            self._current_attribute = None
        elif isinstance(value, objects.CurrentAttribute):
            self._current_attribute = value
        else:
            raise TypeError(
                "The current attribute must be a CurrentAttribute object."
            )

    @property
    def new_attribute(self):
        if self._new_attribute:
            return self._new_attribute
        return None

    @new_attribute.setter
    def new_attribute(self, value):
        if value is None:
            self._new_attribute = None
        elif isinstance(value, objects.NewAttribute):
            self._new_attribute = value
        else:
            raise TypeError(
                "The new attribute must be a NewAttribute object."
            )

    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the ModifyAttribute request payload and decode
        it into its constituent part.

        Args:
            input_buffer (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidKmipEncoding: Raised if fields are missing from the
                encoding.
        """
        super(ModifyAttributeRequestPayload, self).read(
            input_buffer,
            kmip_version=kmip_version
        )
        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            self._unique_identifier = None

        if kmip_version < enums.KMIPVersion.KMIP_2_0:
            if self.is_tag_next(enums.Tags.ATTRIBUTE, local_buffer):
                self._attribute = objects.Attribute()
                self._attribute.read(local_buffer, kmip_version=kmip_version)
            else:
                raise exceptions.InvalidKmipEncoding(
                    "The ModifyAttribute request payload encoding is missing "
                    "the attribute field."
                )
        else:
            if self.is_tag_next(enums.Tags.CURRENT_ATTRIBUTE, local_buffer):
                self._current_attribute = objects.CurrentAttribute()
                self._current_attribute.read(
                    local_buffer,
                    kmip_version=kmip_version
                )
            else:
                self._current_attribute = None

            if self.is_tag_next(enums.Tags.NEW_ATTRIBUTE, local_buffer):
                self._new_attribute = objects.NewAttribute()
                self._new_attribute.read(
                    local_buffer,
                    kmip_version=kmip_version
                )
            else:
                raise exceptions.InvalidKmipEncoding(
                    "The ModifyAttribute request payload encoding is missing "
                    "the new attribute field."
                )

        self.is_oversized(local_buffer)

    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the ModifyAttribute request payload to a
        stream.

        Args:
            output_buffer (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidField: Raised if a required field is missing from the
                payload object.
        """
        local_buffer = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_buffer,
                kmip_version=kmip_version
            )

        if kmip_version < enums.KMIPVersion.KMIP_2_0:
            if self._attribute:
                self._attribute.write(
                    local_buffer,
                    kmip_version=kmip_version
                )
            else:
                raise exceptions.InvalidField(
                    "The ModifyAttribute request payload is missing the "
                    "attribute field."
                )
        else:
            if self._current_attribute:
                self._current_attribute.write(
                    local_buffer,
                    kmip_version=kmip_version
                )

            if self._new_attribute:
                self._new_attribute.write(
                    local_buffer,
                    kmip_version=kmip_version
                )
            else:
                raise exceptions.InvalidField(
                    "The ModifyAttribute request payload is missing the new "
                    "attribute field."
                )

        self.length = local_buffer.length()
        super(ModifyAttributeRequestPayload, self).write(
            output_buffer,
            kmip_version=kmip_version
        )
        output_buffer.write(local_buffer.buffer)

    def __repr__(self):
        args = [
            "unique_identifier='{}'".format(self.unique_identifier),
            "attribute={}".format(
                repr(self.attribute) if self.attribute else None
            ),
            "current_attribute={}".format(
                repr(self.current_attribute) if self.current_attribute
                else None
            ),
            "new_attribute={}".format(
                repr(self.new_attribute) if self.new_attribute else None
            )
        ]
        return "ModifyAttributeRequestPayload({})".format(", ".join(args))

    def __str__(self):
        return str(
            {
                "unique_identifier": self.unique_identifier,
                "attribute": str(self.attribute) if self.attribute else None,
                "current_attribute": str(
                    self.current_attribute
                ) if self.current_attribute else None,
                "new_attribute": str(
                    self.new_attribute
                ) if self.new_attribute else None
            }
        )

    def __eq__(self, other):
        if isinstance(other, ModifyAttributeRequestPayload):
            if self.unique_identifier != other.unique_identifier:
                return False
            elif self.attribute != other.attribute:
                return False
            elif self.current_attribute != other.current_attribute:
                return False
            elif self.new_attribute != other.new_attribute:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, ModifyAttributeRequestPayload):
            return not self.__eq__(other)
        else:
            return NotImplemented


class ModifyAttributeResponsePayload(base.ResponsePayload):
    """
    A response payload for the ModifyAttribute operation.

    Attributes:
        unique_identifier: The unique ID of the object on which the attribute
            was set.
        attribute: The newly modified attribute. Used in KMIP 1.0 - 1.4.
    """

    def __init__(self, unique_identifier=None, attribute=None):
        """
        Construct a ModifyAttribute response payload.

        Args:
            unique_identifier (string): The unique ID of the object on
                which the attribute was set. Defaults to None. Required for
                read/write.
            attribute (struct): An Attribute object representing the newly
                modified attribute. Optional, defaults to None. Used in KMIP
                1.0 - 1.4. Required for read/write.
        """
        super(ModifyAttributeResponsePayload, self).__init__()

        self._unique_identifier = None
        self._attribute = None

        self.unique_identifier = unique_identifier
        self.attribute = attribute

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("The unique identifier must be a string.")

    @property
    def attribute(self):
        if self._attribute:
            return self._attribute
        return None

    @attribute.setter
    def attribute(self, value):
        if value is None:
            self._attribute = None
        elif isinstance(value, objects.Attribute):
            self._attribute = value
        else:
            raise TypeError("The attribute must be an Attribute object.")

    def read(self, input_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the ModifyAttribute response payload and decode
        it into its constituent parts.

        Args:
            input_buffer (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (enum): A KMIPVersion enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidKmipEncoding: Raised if any required fields are missing
                from the encoding.
        """
        super(ModifyAttributeResponsePayload, self).read(
            input_buffer,
            kmip_version=kmip_version
        )
        local_buffer = utils.BytearrayStream(input_buffer.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_buffer):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            raise exceptions.InvalidKmipEncoding(
                "The ModifyAttribute response payload encoding is missing the "
                "unique identifier field."
            )

        if kmip_version < enums.KMIPVersion.KMIP_2_0:
            if self.is_tag_next(enums.Tags.ATTRIBUTE, local_buffer):
                self._attribute = objects.Attribute()
                self._attribute.read(local_buffer, kmip_version=kmip_version)
            else:
                raise exceptions.InvalidKmipEncoding(
                    "The ModifyAttribute response payload encoding is missing "
                    "the attribute field."
                )

        self.is_oversized(local_buffer)

    def write(self, output_buffer, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the ModifyAttribute response payload to a
        buffer.

        Args:
            output_buffer (buffer): A data buffer in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (enum): A KMIPVersion enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            InvalidField: Raised if a required field is missing from the
                payload object.
        """
        local_buffer = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_buffer,
                kmip_version=kmip_version
            )
        else:
            raise exceptions.InvalidField(
                "The ModifyAttribute response payload is missing the unique "
                "identifier field."
            )

        if kmip_version < enums.KMIPVersion.KMIP_2_0:
            if self._attribute:
                self._attribute.write(local_buffer, kmip_version=kmip_version)
            else:
                raise exceptions.InvalidField(
                    "The ModifyAttribute response payload is missing the "
                    "attribute field."
                )

        self.length = local_buffer.length()
        super(ModifyAttributeResponsePayload, self).write(
            output_buffer,
            kmip_version=kmip_version
        )
        output_buffer.write(local_buffer.buffer)

    def __repr__(self):
        args = [
            "unique_identifier='{}'".format(self.unique_identifier),
            "attribute={}".format(
                repr(self.attribute) if self.attribute else None
            )
        ]
        return "ModifyAttributeResponsePayload({})".format(", ".join(args))

    def __str__(self):
        return str(
            {
                "unique_identifier": self.unique_identifier,
                "attribute": str(self.attribute) if self.attribute else None
            }
        )

    def __eq__(self, other):
        if isinstance(other, ModifyAttributeResponsePayload):
            if self.unique_identifier != other.unique_identifier:
                return False
            elif self.attribute != other.attribute:
                return False
            else:
                return True
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, ModifyAttributeResponsePayload):
            return not self.__eq__(other)
        return NotImplemented



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from six.moves import xrange

from kmip.core import enums
from kmip.core.messages.contents import ProtocolVersion
from kmip.core.messages.payloads import base
from kmip.core.utils import BytearrayStream


class DiscoverVersionsRequestPayload(base.RequestPayload):

    def __init__(self, protocol_versions=None):
        super(DiscoverVersionsRequestPayload, self).__init__()

        if protocol_versions is None:
            self.protocol_versions = list()
        else:
            self.protocol_versions = protocol_versions

        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(DiscoverVersionsRequestPayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        while(self.is_tag_next(enums.Tags.PROTOCOL_VERSION, tstream)):
            protocol_version = ProtocolVersion()
            protocol_version.read(tstream, kmip_version=kmip_version)
            self.protocol_versions.append(protocol_version)

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        for protocol_version in self.protocol_versions:
            protocol_version.write(tstream, kmip_version=kmip_version)

        self.length = tstream.length()
        super(DiscoverVersionsRequestPayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        self.__validate()

    def __validate(self):
        if isinstance(self.protocol_versions, list):
            for i in xrange(len(self.protocol_versions)):
                protocol_version = self.protocol_versions[i]
                if not isinstance(protocol_version, ProtocolVersion):
                    msg = "invalid protocol version ({0} in list)".format(i)
                    msg += "; expected {0}, received {1}".format(
                        ProtocolVersion, protocol_version)
                    raise TypeError(msg)
        else:
            msg = "invalid protocol versions list"
            msg += "; expected {0}, received {1}".format(
                list, self.protocol_versions)
            raise TypeError(msg)


class DiscoverVersionsResponsePayload(base.ResponsePayload):

    def __init__(self, protocol_versions=None):
        super(DiscoverVersionsResponsePayload, self).__init__()

        if protocol_versions is None:
            self.protocol_versions = list()
        else:
            self.protocol_versions = protocol_versions

        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(DiscoverVersionsResponsePayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        while(self.is_tag_next(enums.Tags.PROTOCOL_VERSION, tstream)):
            protocol_version = ProtocolVersion()
            protocol_version.read(tstream, kmip_version=kmip_version)
            self.protocol_versions.append(protocol_version)

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        for protocol_version in self.protocol_versions:
            protocol_version.write(tstream, kmip_version=kmip_version)

        self.length = tstream.length()
        super(DiscoverVersionsResponsePayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        self.__validate()

    def __validate(self):
        if isinstance(self.protocol_versions, list):
            for i in xrange(len(self.protocol_versions)):
                protocol_version = self.protocol_versions[i]
                if not isinstance(protocol_version, ProtocolVersion):
                    msg = "invalid protocol version ({0} in list)".format(i)
                    msg += "; expected {0}, received {1}".format(
                        ProtocolVersion, protocol_version)
                    raise TypeError(msg)
        else:
            msg = "invalid protocol versions list"
            msg += "; expected {0}, received {1}".format(
                list, self.protocol_versions)
            raise TypeError(msg)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import six
import ssl


@six.add_metaclass(abc.ABCMeta)
class AuthenticationSuite(object):
    """
    An authentication suite used to establish secure network connections.

    Acts as the base of the suite hierarchy.
    """

    # OpenSSL cipher suites
    # Explicitly listed suites for Basic and TLSv1.2 authentication for KMIP
    # profiles.
    #
    # Obtained from:
    #     https://www.openssl.org/docs/man1.1.0/apps/ciphers.html
    #     https://www.openssl.org/docs/man1.0.2/apps/ciphers.html
    openssl_cipher_suite_map = {
        # TLS v1.2 cipher suites
        'TLS_RSA_WITH_AES_256_CBC_SHA256':         'AES256-SHA256',
        'TLS_RSA_WITH_AES_128_CBC_SHA256':         'AES128-SHA256',
        'TLS_DH_DSS_WITH_AES_128_CBC_SHA256':      'DH-DSS-AES128-SHA256',
        'TLS_DH_RSA_WITH_AES_128_CBC_SHA256':      'DH-RSA-AES128-SHA256',
        'TLS_DHE_DSS_WITH_AES_128_CBC_SHA256':     'DHE-DSS-AES128-SHA256',
        'TLS_DHE_RSA_WITH_AES_128_CBC_SHA256':     'DHE-RSA-AES128-SHA256',
        'TLS_DH_DSS_WITH_AES_256_CBC_SHA256':      'DH-DSS-AES256-SHA256',
        'TLS_DH_RSA_WITH_AES_256_CBC_SHA256':      'DH-RSA-AES256-SHA256',
        'TLS_DHE_DSS_WITH_AES_256_CBC_SHA256':     'DHE-DSS-AES256-SHA256',
        'TLS_DHE_RSA_WITH_AES_256_CBC_SHA256':     'DHE-RSA-AES256-SHA256',
        'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256': 'ECDHE-ECDSA-AES128-SHA256',
        'TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384': 'ECDHE-ECDSA-AES256-SHA384',
        'TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256':   'ECDHE-RSA-AES128-SHA256',
        'TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384':   'ECDHE-RSA-AES256-SHA384',
        'TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256':
            'ECDHE-ECDSA-AES128-GCM-SHA256',
        'TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384':
            'ECDHE-ECDSA-AES256-GCM-SHA384',
        'TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256':  'ECDH-ECDSA-AES128-SHA256',
        'TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384':  'ECDH-ECDSA-AES256-SHA384',
        'TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256':    'ECDH-RSA-AES128-SHA256',
        'TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384':    'ECDH-RSA-AES256-SHA384',

        # AES ciphersuites from RFC3268, extending TLS v1.0
        'TLS_RSA_WITH_AES_128_CBC_SHA':     'AES128-SHA',
        'TLS_RSA_WITH_AES_256_CBC_SHA':     'AES256-SHA',
        'TLS_DH_DSS_WITH_AES_128_CBC_SHA':  'DH-DSS-AES128-SHA',
        'TLS_DH_RSA_WITH_AES_128_CBC_SHA':  'DH-RSA-AES128-SHA',
        'TLS_DHE_DSS_WITH_AES_128_CBC_SHA': 'DHE-DSS-AES128-SHA',
        'TLS_DHE_RSA_WITH_AES_128_CBC_SHA': 'DHE-RSA-AES128-SHA',
        'TLS_DH_DSS_WITH_AES_256_CBC_SHA':  'DH-DSS-AES256-SHA',
        'TLS_DH_RSA_WITH_AES_256_CBC_SHA':  'DH-RSA-AES256-SHA',
        'TLS_DHE_DSS_WITH_AES_256_CBC_SHA': 'DHE-DSS-AES256-SHA',
        'TLS_DHE_RSA_WITH_AES_256_CBC_SHA': 'DHE-RSA-AES256-SHA',

        # Elliptic curve cipher suites.
        'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA': 'ECDHE-ECDSA-AES128-SHA',
        'TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA':   'ECDHE-RSA-AES128-SHA',

        # Pre shared keying (PSK) cipheruites
        'TLS_PSK_WITH_AES_128_CBC_SHA': 'PSK-AES128-CBC-SHA',
        'TLS_PSK_WITH_AES_256_CBC_SHA': 'PSK-AES256-CBC-SHA',

        # No OpenSSL support
        'TLS_DHE_PSK_WITH_AES_128_CBC_SHA': None,
        'TLS_DHE_PSK_WITH_AES_256_CBC_SHA': None,
        'TLS_RSA_PSK_WITH_AES_128_CBC_SHA': None,
        'TLS_RSA_PSK_WITH_AES_256_CBC_SHA': None
    }

    _default_cipher_suites = []

    @abc.abstractmethod
    def __init__(self, cipher_suites=None):
        """
        Create an AuthenticationSuite object.

        Args:
            cipher_suites (list): A list of strings representing the names of
                cipher suites to use. Overrides the default set of cipher
                suites. Optional, defaults to None.
        """
        self._custom_suites = []

        # Compose a unique list of custom cipher suites if any were provided.
        # Translate each suite name into its corresponding OpenSSL suite name,
        # allowing for both specification and OpenSSL suite names in the
        # provided list.
        if cipher_suites:
            for cipher_suite in cipher_suites:
                if cipher_suite in self.openssl_cipher_suite_map.keys():
                    suite = self.openssl_cipher_suite_map.get(cipher_suite)
                    if suite:
                        self._custom_suites.append(suite)
                elif cipher_suite in self.openssl_cipher_suite_map.values():
                    if cipher_suite:
                        self._custom_suites.append(cipher_suite)
            self._custom_suites = list(set(self._custom_suites))

        # Filter the custom suites to only include those from the default
        # cipher suite list (provided for each subclass authentication suite).
        # If no custom suites were specified, use the default cipher suites.
        suites = []
        if self._custom_suites:
            for suite in self._custom_suites:
                if suite in self._default_cipher_suites:
                    suites.append(suite)
        else:
            suites = self._default_cipher_suites

        self._cipher_suites = ':'.join(suites)
        if self._cipher_suites == '':
            self._cipher_suites = ':'.join(self._default_cipher_suites)

    @property
    def protocol(self):
        """
        Get the authentication suite protocol.

        Returns:
            int: The value of the ssl.PROTOCOL_* setting for this suite.
        """
        return self._protocol

    @property
    def ciphers(self):
        """
        Get the authentication suite cipher string.

        Returns:
            string: A colon delimited string listing the valid ciphers for
                the suite protocol.
        """
        return self._cipher_suites


class BasicAuthenticationSuite(AuthenticationSuite):
    """
    An authentication suite used to establish secure network connections.

    Supports TLS 1.0 and a subset of TLS 1.0 compliant cipher suites, defined
    in NIST 800-57, as defined by the KMIP specification.
    """

    _default_cipher_suites = [
        'AES128-SHA',
        'DES-CBC3-SHA',
        'AES256-SHA',
        'DHE-DSS-DES-CBC3-SHA',
        'DHE-RSA-DES-CBC3-SHA',
        'DH-DSS-AES128-SHA',
        'DH-RSA-AES128-SHA',
        'DHE-DSS-AES128-SHA',
        'DHE-RSA-AES128-SHA',
        'DH-RSA-AES256-SHA',
        'DHE-DSS-AES256-SHA',
        'DHE-RSA-AES256-SHA'
    ]

    def __init__(self, cipher_suites=None):
        """
        Create a BasicAuthenticationSuite object.

        Args:
            cipher_suites (list): A list of strings representing the names of
                cipher suites to use. Overrides the default set of cipher
                suites. Optional, defaults to None.
        """
        super(BasicAuthenticationSuite, self).__init__(cipher_suites)
        self._protocol = ssl.PROTOCOL_TLSv1


class TLS12AuthenticationSuite(AuthenticationSuite):
    """
    An authentication suite used to establish secure network connections.

    Supports TLS 1.2 and a subset of TLS 1.2 compliant cipher suites, defined
    in NIST 800-57, as defined by the KMIP specification.
    """

    _default_cipher_suites = [
        'AES128-SHA256',
        'AES256-SHA256',
        'DH-DSS-AES256-SHA256',
        'DH-DSS-AES128-SHA256',
        'DH-RSA-AES128-SHA256',
        'DHE-DSS-AES128-SHA256',
        'DHE-RSA-AES128-SHA256',
        'DH-DSS-AES256-SHA256',
        'DH-RSA-AES256-SHA256',
        'DHE-DSS-AES256-SHA256',
        'DHE-RSA-AES256-SHA256',
        'ECDH-ECDSA-AES128-SHA256',
        'ECDH-ECDSA-AES256-SHA256',
        'ECDHE-ECDSA-AES128-SHA256',
        'ECDHE-ECDSA-AES256-SHA384',
        'ECDH-RSA-AES128-SHA256',
        'ECDH-RSA-AES256-SHA384',
        'ECDHE-RSA-AES128-SHA256',
        'ECDHE-RSA-AES256-SHA384',
        'ECDHE-ECDSA-AES128-GCM-SHA256',
        'ECDHE-ECDSA-AES256-GCM-SHA384',
        'ECDHE-ECDSA-AES128-SHA256',
        'ECDHE-ECDSA-AES256-SHA384'
    ]

    def __init__(self, cipher_suites=None):
        """
        Create a TLS12AuthenticationSuite object.

        Args:
            cipher_suites (list): A list of strings representing the names of
                cipher suites to use. Overrides the default set of cipher
                suites. Optional, defaults to None.
        """
        super(TLS12AuthenticationSuite, self).__init__(cipher_suites)
        self._protocol = ssl.PROTOCOL_TLSv1_2



# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils
from kmip.pie import client
from kmip.pie import objects


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(enums.Operation.SIGNATURE_VERIFY)
    opts, args = parser.parse_args(sys.argv[1:])
    config = opts.config

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        # Create keys to use for derivation
        try:
            signing_key_id = client.register(
                objects.PublicKey(
                    enums.CryptographicAlgorithm.RSA,
                    1120,
                    (
                        b'\x30\x81\x89\x02\x81\x81\x00\xac\x13\xd9\xfd\xae\x7b'
                        b'\x73\x35\xb6\x9c\xd9\x85\x67\xe9\x64\x7d\x99\xbf\x37'
                        b'\x3a\x9e\x05\xce\x34\x35\xd6\x64\x65\xf3\x28\xb7\xf7'
                        b'\x33\x4b\x79\x2a\xee\x7e\xfa\x04\x4e\xbc\x4c\x7a\x30'
                        b'\xb2\x1a\x5d\x7a\x89\xcd\xb3\xa3\x0d\xfc\xd9\xfe\xe9'
                        b'\x99\x5e\x09\x41\x5e\xdc\x0b\xf9\xe5\xb4\xc3\xf7\x4f'
                        b'\xf5\x3f\xb4\xd2\x94\x41\xbf\x1b\x7e\xd6\xcb\xdd\x4a'
                        b'\x47\xf9\x25\x22\x69\xe1\x64\x6f\x6c\x1a\xee\x05\x14'
                        b'\xe9\x3f\x6c\xb9\xdf\x71\xd0\x6c\x06\x0a\x21\x04\xb4'
                        b'\x7b\x72\x60\xac\x37\xc1\x06\x86\x1d\xc7\x8c\xa5\xa2'
                        b'\x5f\xaa\x9c\xb2\xe3\x02\x03\x01\x00\x01'
                    ),
                    masks=[
                        enums.CryptographicUsageMask.SIGN,
                        enums.CryptographicUsageMask.VERIFY
                    ]
                )
            )
            logger.info("Successfully created a new signing key.")
            logger.info("Signing Key ID: {0}".format(signing_key_id))
        except Exception as e:
            logger.error(e)
            sys.exit(-1)

        # Activate the signing key.
        try:
            client.activate(signing_key_id)
            logger.info(
                "Signing key {0} has been activated.".format(signing_key_id)
            )
        except Exception as e:
            logger.error(e)
            sys.exit(-1)

        # Verify a valid signature.
        try:
            result = client.signature_verify(
                (
                    b'\xe1\xc0\xf9\x8d\x53\xf8\xf8\xb1\x41\x90\x57\xd5\xb9\xb1'
                    b'\x0b\x07\xfe\xea\xec\x32\xc0\x46\x3a\x4d\x68\x38\x2f\x53'
                    b'\x1b\xa1\xd6\xcf\xe4\xed\x38\xa2\x69\x4a\x34\xb9\xc8\x05'
                    b'\xad\xf0\x72\xff\xbc\xeb\xe2\x1d\x8d\x4b\x5c\x0e\x8c\x33'
                    b'\x45\x2d\xd8\xf9\xc9\xbf\x45\xd1\xe6\x33\x75\x11\x33\x58'
                    b'\x82\x29\xd2\x93\xc6\x49\x6b\x7c\x98\x3c\x2c\x72\xbd\x21'
                    b'\xd3\x39\x27\x2d\x78\x28\xb0\xd0\x9d\x01\x0b\xba\xd3\x18'
                    b'\xd9\x98\xf7\x04\x79\x67\x33\x8a\xce\xfd\x01\xe8\x74\xac'
                    b'\xe5\xf8\x6d\x2a\x60\xf3\xb3\xca\xe1\x3f\xc5\xc6\x65\x08'
                    b'\xcf\xb7\x23\x78\xfd\xd6\xc8\xde\x24\x97\x65\x10\x3c\xe8'
                    b'\xfe\x7c\xd3\x3a\xd0\xef\x16\x86\xfe\xb2\x5e\x6a\x35\xfb'
                    b'\x64\xe0\x96\xa4'
                ),
                (
                    b'\x01\xf6\xe5\xff\x04\x22\x1a\xdc\x6c\x2f\x22\xa7\x61\x05'
                    b'\x3b\xc4\x73\x27\x65\xdd\xdc\x3f\x76\x56\xd0\xd1\x22\xad'
                    b'\x3b\x8a\x4e\x4f\x8f\xe5\x5b\xd0\xc0\x9e\xb1\x07\x80\xa1'
                    b'\x39\xcd\xa9\x32\x34\xef\x98\x8f\xe2\x50\x20\x1e\xb2\xfe'
                    b'\xbd\x08\xb6\xee\x85\xd7\x0d\x16\x05\xa5\xba\x56\x85\x21'
                    b'\x52\x99\xf0\x74\xc8\x0b\xaf\xf8\x1e\x2c\xa3\x10\x7d\xa9'
                    b'\x17\x5c\x2f\x5a\x7c\x6b\x60\xea\xa2\x8a\x75\x8c\xa9\x34'
                    b'\xf2\xff\x16\x98\x8f\xe8\x5f\xf8\x41\x57\xd9\x51\x44\x8a'
                    b'\x85\xec\x1e\xd1\x71\xf9\xef\x8b\xb8\xa1\x0c\xfa\x14\x7b'
                    b'\x7e\xf8'
                ),
                uid=signing_key_id,
                cryptographic_parameters={
                    'cryptographic_algorithm':
                        enums.CryptographicAlgorithm.RSA,
                    'hashing_algorithm': enums.HashingAlgorithm.SHA_1,
                    'padding_method': enums.PaddingMethod.PSS
                },
            )
            if result == enums.ValidityIndicator.VALID:
                logger.info("Example 1: The signature is valid.")
            elif result == enums.ValidityIndicator.INVALID:
                logger.info("Example 1: The signature is invalid.")
            else:
                logger.info(
                    "Example 1: The signature validity is undetermined."
                )
        except Exception as e:
            logger.error(e)

        # Verify an invalid signature.
        try:
            result = client.signature_verify(
                b'This message is invalid.',
                (
                    b'\x01\xf6\xe5\xff\x04\x22\x1a\xdc\x6c\x2f\x22\xa7\x61\x05'
                    b'\x3b\xc4\x73\x27\x65\xdd\xdc\x3f\x76\x56\xd0\xd1\x22\xad'
                    b'\x3b\x8a\x4e\x4f\x8f\xe5\x5b\xd0\xc0\x9e\xb1\x07\x80\xa1'
                    b'\x39\xcd\xa9\x32\x34\xef\x98\x8f\xe2\x50\x20\x1e\xb2\xfe'
                    b'\xbd\x08\xb6\xee\x85\xd7\x0d\x16\x05\xa5\xba\x56\x85\x21'
                    b'\x52\x99\xf0\x74\xc8\x0b\xaf\xf8\x1e\x2c\xa3\x10\x7d\xa9'
                    b'\x17\x5c\x2f\x5a\x7c\x6b\x60\xea\xa2\x8a\x75\x8c\xa9\x34'
                    b'\xf2\xff\x16\x98\x8f\xe8\x5f\xf8\x41\x57\xd9\x51\x44\x8a'
                    b'\x85\xec\x1e\xd1\x71\xf9\xef\x8b\xb8\xa1\x0c\xfa\x14\x7b'
                    b'\x7e\xf8'
                ),
                uid=signing_key_id,
                cryptographic_parameters={
                    'cryptographic_algorithm':
                        enums.CryptographicAlgorithm.RSA,
                    'hashing_algorithm': enums.HashingAlgorithm.SHA_1,
                    'padding_method': enums.PaddingMethod.PSS
                },
            )
            if result == enums.ValidityIndicator.VALID:
                logger.info("Example 2: The signature is valid.")
            elif result == enums.ValidityIndicator.INVALID:
                logger.info("Example 2: The signature is invalid.")
            else:
                logger.info(
                    "Example 2: The signature validity is undetermined."
                )
        except Exception as e:
            logger.error(e)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from testtools import TestCase
from mock import MagicMock
from mock import Mock

from kmip.core.config_helper import ConfigHelper


class TestConfigHelper(TestCase):

    def setUp(self):
        def side_effect(arg1, arg2):
            if arg1 == 'conf_test_section' and arg2 == 'conf_test_option':
                return 'conf_test_value'
            elif arg1 == 'conf_test_section' and arg2 == 'conf_null_option':
                return ConfigHelper.NONE_VALUE
            else:
                raise configparser.NoSectionError
        super(TestConfigHelper, self).setUp()
        self.config_helper = ConfigHelper()
        self.config_helper.conf = MagicMock()
        self.config_helper.conf.get = Mock(side_effect=side_effect)

    def tearDown(self):
        super(TestConfigHelper, self).tearDown()

    def test_get_valid_value_null_input(self):
        value = self.config_helper.get_valid_value(None, None, None, None)
        self.assertEqual(None, value)

    def test_get_valid_value_direct_value_is_none(self):
        value = self.config_helper.get_valid_value(ConfigHelper.NONE_VALUE,
                                                   'conf_test_section',
                                                   'conf_test_option',
                                                   'test_default_option')
        self.assertFalse(self.config_helper.conf.get.called)
        self.assertEqual(None, value)

    def test_get_valid_value_config_value_is_none(self):
        value = self.config_helper.get_valid_value(None,
                                                   'conf_test_section',
                                                   'conf_null_option',
                                                   'test_default_option')
        self.assertTrue(self.config_helper.conf.get.called)
        self.config_helper.conf.get.assert_called_with('conf_test_section',
                                                       'conf_null_option')
        self.assertEqual(None, value)

    def test_get_valid_value_returns_direct(self):
        value = self.config_helper.get_valid_value('test_direct_value',
                                                   'conf_test_section',
                                                   'conf_test_option',
                                                   'test_default_value')
        self.assertFalse(self.config_helper.conf.get.called)
        self.assertEqual('test_direct_value', value)

    def test_get_valid_value_returns_conf_value(self):
        value = self.config_helper.get_valid_value(None,
                                                   'conf_test_section',
                                                   'conf_test_option',
                                                   'test_default_value')
        self.assertTrue(self.config_helper.conf.get.called)
        self.config_helper.conf.get.assert_called_with('conf_test_section',
                                                       'conf_test_option')
        self.assertEqual(value, 'conf_test_value')

    def test_get_valid_value_returns_default(self):
        value = self.config_helper.get_valid_value(None,
                                                   'invalid_section',
                                                   'invalid_option',
                                                   'test_default_value')
        self.assertTrue(self.config_helper.conf.get.called)
        self.config_helper.conf.get.assert_called_with('invalid_section',
                                                       'invalid_option')
        self.assertEqual(value, 'test_default_value')



#!/usr/bin/env python
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import setuptools

# Dynamically set __version__
version_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "kmip",
    "version.py"
)
with open(version_path, 'r') as f:
    m = re.search(
        r"^__version__ = \"(\d+\.\d+\..*)\"$",
        f.read(),
        re.MULTILINE
    )
    __version__ = m.group(1)

setuptools.setup(
    name='PyKMIP',
    version=__version__,
    description='KMIP library',
    keywords='KMIP',
    author='Peter Hamilton',
    author_email='peter.hamilton@jhuapl.edu',
    url='https://github.com/OpenKMIP/PyKMIP',
    license='Apache License, Version 2.0',
    packages=setuptools.find_packages(exclude=["kmip.tests", "kmip.tests.*"]),
    package_data={'kmip': ['kmipconfig.ini', 'logconfig.ini'],
                  'kmip.demos': ['certs/server.crt', 'certs/server.key']},
    entry_points={
        'console_scripts':[
            'pykmip-server = kmip.services.server.server:main'
        ]
    },
    install_requires=[
        "cryptography",
        "enum-compat",
        "requests",
        "six",
        "sqlalchemy"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import enums
from kmip.core.enums import AttributeType
from kmip.core.enums import CredentialType
from kmip.core.enums import CryptographicAlgorithm
from kmip.core.enums import CryptographicUsageMask as UsageMaskEnum
from kmip.core.enums import Operation
from kmip.core.enums import ResultStatus
from kmip.core.enums import NameType

from kmip.demos import utils

from kmip.core.factories.attributes import AttributeFactory
from kmip.core.factories.credentials import CredentialFactory

from kmip.core.attributes import Name
from kmip.core.attributes import CryptographicUsageMask

from kmip.core.objects import TemplateAttribute
from kmip.core.objects import Attribute

from kmip.services.kmip_client import KMIPProxy

import logging
import sys


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(Operation.CREATE_KEY_PAIR)
    opts, args = parser.parse_args(sys.argv[1:])

    username = opts.username
    password = opts.password
    config = opts.config
    algorithm = opts.algorithm
    length = opts.length
    name = opts.name

    # Exit early if the arguments are not specified
    if algorithm is None:
        logger.error('No algorithm provided, exiting early from demo')
        sys.exit()
    if length is None:
        logger.error("No key length provided, exiting early from demo")
        sys.exit()
    if name is None:
        logger.error("No key name provided, exiting early from demo")
        sys.exit()

    attribute_type = AttributeType.CRYPTOGRAPHIC_ALGORITHM
    algorithm_enum = getattr(CryptographicAlgorithm, algorithm, None)

    if algorithm_enum is None:
        logger.error("Invalid algorithm specified; exiting early from demo")
        sys.exit()

    attribute_factory = AttributeFactory()
    credential_factory = CredentialFactory()

    # Build the KMIP server account credentials
    # TODO (peter-hamilton) Move up into KMIPProxy
    if (username is None) and (password is None):
        credential = None
    else:
        credential_type = CredentialType.USERNAME_AND_PASSWORD
        credential_value = {'Username': username,
                            'Password': password}
        credential = credential_factory.create_credential(credential_type,
                                                          credential_value)
    # Build the client and connect to the server
    client = KMIPProxy(config=config, config_file=opts.config_file)
    client.open()

    algorithm_obj = attribute_factory.create_attribute(attribute_type,
                                                       algorithm_enum)

    name_value = Name.NameValue(name)
    name = Attribute.AttributeName('Name')
    name_type = Name.NameType(NameType.UNINTERPRETED_TEXT_STRING)
    value = Name(name_value=name_value, name_type=name_type)
    name = Attribute(attribute_name=name, attribute_value=value)

    usage_mask = Attribute.AttributeName('Cryptographic Usage Mask')
    value = CryptographicUsageMask(
        UsageMaskEnum.ENCRYPT.value | UsageMaskEnum.DECRYPT.value)
    usage_mask = Attribute(attribute_name=usage_mask, attribute_value=value)

    attribute_type = AttributeType.CRYPTOGRAPHIC_LENGTH
    length_obj = attribute_factory.create_attribute(attribute_type,
                                                    length)

    attributes = [algorithm_obj, length_obj, name, usage_mask]

    if opts.operation_policy_name is not None:
        opn = attribute_factory.create_attribute(
            enums.AttributeType.OPERATION_POLICY_NAME,
            opts.operation_policy_name
        )
        attributes.append(opn)

    common = TemplateAttribute(
        attributes=attributes,
        tag=enums.Tags.COMMON_TEMPLATE_ATTRIBUTE
    )
    private = TemplateAttribute(
        attributes=attributes,
        tag=enums.Tags.PRIVATE_KEY_TEMPLATE_ATTRIBUTE
    )
    public = TemplateAttribute(
        attributes=attributes,
        tag=enums.Tags.PUBLIC_KEY_TEMPLATE_ATTRIBUTE
    )

    # Create the SYMMETRIC_KEY object
    result = client.create_key_pair(
        common_template_attribute=common,
        private_key_template_attribute=private,
        public_key_template_attribute=public
    )
    client.close()

    # Display operation results
    logger.info('create_key_pair() result status: {0}'.format(
        result.result_status.value))

    if result.result_status.value == ResultStatus.SUCCESS:
        logger.info('created private key UUID: {0}'.format(
            result.private_key_uuid))
        logger.info('created public key UUID: {0}'.format(
            result.public_key_uuid))

        if result.private_key_template_attribute is not None:
            logger.info('private key template attribute:')
            utils.log_template_attribute(
                logger, result.private_key_template_attribute)

        if result.public_key_template_attribute is not None:
            logger.info('public key template attribute:')
            utils.log_template_attribute(
                logger, result.public_key_template_attribute)
    else:
        logger.info('create() result reason: {0}'.format(
            result.result_reason.value))
        logger.info('create() result message: {0}'.format(
            result.result_message.value))



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core.attributes import CertificateType
from kmip.core import enums
from kmip.core.misc import CertificateValue
from kmip.core.secrets import Certificate
from kmip.core.utils import BytearrayStream


class TestCertificate(TestCase):
    """
    A test suite for the Certificate class.
    """

    def setUp(self):
        super(TestCertificate, self).setUp()

        self.certificate_type_a = None
        self.certificate_type_b = enums.CertificateType.X_509

        # Encodings obtained from Section 13.2 of KMIP 1.1 Test Cases document.
        self.certificate_value_a = None
        self.certificate_value_b = (
            b'\x30\x82\x03\x12\x30\x82\x01\xFA\xA0\x03\x02\x01\x02\x02\x01\x01'
            b'\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05\x00\x30'
            b'\x3B\x31\x0B\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D'
            b'\x30\x0B\x06\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30'
            b'\x0C\x06\x03\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30'
            b'\x0B\x06\x03\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x1E\x17\x0D'
            b'\x31\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x17\x0D\x32'
            b'\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x30\x3B\x31\x0B'
            b'\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D\x30\x0B\x06'
            b'\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30\x0C\x06\x03'
            b'\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30\x0B\x06\x03'
            b'\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x82\x01\x22\x30\x0D\x06'
            b'\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03\x82\x01\x0F'
            b'\x00\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42'
            b'\x49\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00'
            b'\x3A\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00'
            b'\x2C\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D'
            b'\x75\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7'
            b'\xEC\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F'
            b'\xA2\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4'
            b'\xBB\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF'
            b'\x8B\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D'
            b'\x57\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC'
            b'\xC8\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7'
            b'\xF4\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D'
            b'\x0C\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE'
            b'\x64\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC'
            b'\xE8\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73'
            b'\xA1\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC'
            b'\x41\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01\xA3'
            b'\x21\x30\x1F\x30\x1D\x06\x03\x55\x1D\x0E\x04\x16\x04\x14\x04\xE5'
            b'\x7B\xD2\xC4\x31\xB2\xE8\x16\xE1\x80\xA1\x98\x23\xFA\xC8\x58\x27'
            b'\x3F\x6B\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05'
            b'\x00\x03\x82\x01\x01\x00\xA8\x76\xAD\xBC\x6C\x8E\x0F\xF0\x17\x21'
            b'\x6E\x19\x5F\xEA\x76\xBF\xF6\x1A\x56\x7C\x9A\x13\xDC\x50\xD1\x3F'
            b'\xEC\x12\xA4\x27\x3C\x44\x15\x47\xCF\xAB\xCB\x5D\x61\xD9\x91\xE9'
            b'\x66\x31\x9D\xF7\x2C\x0D\x41\xBA\x82\x6A\x45\x11\x2F\xF2\x60\x89'
            b'\xA2\x34\x4F\x4D\x71\xCF\x7C\x92\x1B\x4B\xDF\xAE\xF1\x60\x0D\x1B'
            b'\xAA\xA1\x53\x36\x05\x7E\x01\x4B\x8B\x49\x6D\x4F\xAE\x9E\x8A\x6C'
            b'\x1D\xA9\xAE\xB6\xCB\xC9\x60\xCB\xF2\xFA\xE7\x7F\x58\x7E\xC4\xBB'
            b'\x28\x20\x45\x33\x88\x45\xB8\x8D\xD9\xAE\xEA\x53\xE4\x82\xA3\x6E'
            b'\x73\x4E\x4F\x5F\x03\xB9\xD0\xDF\xC4\xCA\xFC\x6B\xB3\x4E\xA9\x05'
            b'\x3E\x52\xBD\x60\x9E\xE0\x1E\x86\xD9\xB0\x9F\xB5\x11\x20\xC1\x98'
            b'\x34\xA9\x97\xB0\x9C\xE0\x8D\x79\xE8\x13\x11\x76\x2F\x97\x4B\xB1'
            b'\xC8\xC0\x91\x86\xC4\xD7\x89\x33\xE0\xDB\x38\xE9\x05\x08\x48\x77'
            b'\xE1\x47\xC7\x8A\xF5\x2F\xAE\x07\x19\x2F\xF1\x66\xD1\x9F\xA9\x4A'
            b'\x11\xCC\x11\xB2\x7E\xD0\x50\xF7\xA2\x7F\xAE\x13\xB2\x05\xA5\x74'
            b'\xC4\xEE\x00\xAA\x8B\xD6\x5D\x0D\x70\x57\xC9\x85\xC8\x39\xEF\x33'
            b'\x6A\x44\x1E\xD5\x3A\x53\xC6\xB6\xB6\x96\xF1\xBD\xEB\x5F\x7E\xA8'
            b'\x11\xEB\xB2\x5A\x7F\x86')

        self.encoding_a = BytearrayStream((
            b'\x42\x00\x13\x01\x00\x00\x00\x18\x42\x00\x1D\x05\x00\x00\x00\x04'
            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x1E\x08\x00\x00\x00'
            b'\x00'))
        self.encoding_b = BytearrayStream((
            b'\x42\x00\x13\x01\x00\x00\x03\x30\x42\x00\x1D\x05\x00\x00\x00\x04'
            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x1E\x08\x00\x00\x03\x16'
            b'\x30\x82\x03\x12\x30\x82\x01\xFA\xA0\x03\x02\x01\x02\x02\x01\x01'
            b'\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05\x00\x30'
            b'\x3B\x31\x0B\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D'
            b'\x30\x0B\x06\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30'
            b'\x0C\x06\x03\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30'
            b'\x0B\x06\x03\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x1E\x17\x0D'
            b'\x31\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x17\x0D\x32'
            b'\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x30\x3B\x31\x0B'
            b'\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D\x30\x0B\x06'
            b'\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30\x0C\x06\x03'
            b'\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30\x0B\x06\x03'
            b'\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x82\x01\x22\x30\x0D\x06'
            b'\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03\x82\x01\x0F'
            b'\x00\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42'
            b'\x49\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00'
            b'\x3A\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00'
            b'\x2C\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D'
            b'\x75\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7'
            b'\xEC\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F'
            b'\xA2\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4'
            b'\xBB\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF'
            b'\x8B\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D'
            b'\x57\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC'
            b'\xC8\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7'
            b'\xF4\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D'
            b'\x0C\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE'
            b'\x64\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC'
            b'\xE8\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73'
            b'\xA1\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC'
            b'\x41\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01\xA3'
            b'\x21\x30\x1F\x30\x1D\x06\x03\x55\x1D\x0E\x04\x16\x04\x14\x04\xE5'
            b'\x7B\xD2\xC4\x31\xB2\xE8\x16\xE1\x80\xA1\x98\x23\xFA\xC8\x58\x27'
            b'\x3F\x6B\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05'
            b'\x00\x03\x82\x01\x01\x00\xA8\x76\xAD\xBC\x6C\x8E\x0F\xF0\x17\x21'
            b'\x6E\x19\x5F\xEA\x76\xBF\xF6\x1A\x56\x7C\x9A\x13\xDC\x50\xD1\x3F'
            b'\xEC\x12\xA4\x27\x3C\x44\x15\x47\xCF\xAB\xCB\x5D\x61\xD9\x91\xE9'
            b'\x66\x31\x9D\xF7\x2C\x0D\x41\xBA\x82\x6A\x45\x11\x2F\xF2\x60\x89'
            b'\xA2\x34\x4F\x4D\x71\xCF\x7C\x92\x1B\x4B\xDF\xAE\xF1\x60\x0D\x1B'
            b'\xAA\xA1\x53\x36\x05\x7E\x01\x4B\x8B\x49\x6D\x4F\xAE\x9E\x8A\x6C'
            b'\x1D\xA9\xAE\xB6\xCB\xC9\x60\xCB\xF2\xFA\xE7\x7F\x58\x7E\xC4\xBB'
            b'\x28\x20\x45\x33\x88\x45\xB8\x8D\xD9\xAE\xEA\x53\xE4\x82\xA3\x6E'
            b'\x73\x4E\x4F\x5F\x03\xB9\xD0\xDF\xC4\xCA\xFC\x6B\xB3\x4E\xA9\x05'
            b'\x3E\x52\xBD\x60\x9E\xE0\x1E\x86\xD9\xB0\x9F\xB5\x11\x20\xC1\x98'
            b'\x34\xA9\x97\xB0\x9C\xE0\x8D\x79\xE8\x13\x11\x76\x2F\x97\x4B\xB1'
            b'\xC8\xC0\x91\x86\xC4\xD7\x89\x33\xE0\xDB\x38\xE9\x05\x08\x48\x77'
            b'\xE1\x47\xC7\x8A\xF5\x2F\xAE\x07\x19\x2F\xF1\x66\xD1\x9F\xA9\x4A'
            b'\x11\xCC\x11\xB2\x7E\xD0\x50\xF7\xA2\x7F\xAE\x13\xB2\x05\xA5\x74'
            b'\xC4\xEE\x00\xAA\x8B\xD6\x5D\x0D\x70\x57\xC9\x85\xC8\x39\xEF\x33'
            b'\x6A\x44\x1E\xD5\x3A\x53\xC6\xB6\xB6\x96\xF1\xBD\xEB\x5F\x7E\xA8'
            b'\x11\xEB\xB2\x5A\x7F\x86\x00\x00'))

    def tearDown(self):
        super(TestCertificate, self).tearDown()

    def test_init(self):
        pass

    def _test_read(self, stream, certificate_type, certificate_value):
        certificate = Certificate()
        certificate.read(stream)

        if certificate_type is None:
            expected = CertificateType()
        else:
            expected = CertificateType(certificate_type)

        observed = certificate.certificate_type

        msg = "certificate type encoding mismatch; "
        msg += "expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed)

        if certificate_value is None:
            expected = CertificateValue()
        else:
            expected = CertificateValue(certificate_value)

        observed = certificate.certificate_value

        msg = "certificate value encoding mismatch; "
        msg += "expected {0}, observed {1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

    def test_read_a(self):
        """
        Test that a Certificate object with no data can be read from a data
        stream.
        """
        self._test_read(self.encoding_a, self.certificate_type_a,
                        self.certificate_value_a)

    def test_read_b(self):
        """
        Test that a Certificate object with data can be read from a data
        stream.
        """
        self._test_read(self.encoding_b, self.certificate_type_b,
                        self.certificate_value_b)

    def _test_write(self, stream, certificate_type, certificate_value):
        certificate = Certificate(
            certificate_type=certificate_type,
            certificate_value=certificate_value)

        expected = stream
        observed = BytearrayStream()

        certificate.write(observed)

        msg = "encoding mismatch;\nexpected:\n{0}\nobserved:\n{1}".format(
            expected, observed)
        self.assertEqual(expected, observed, msg)

    def test_write_a(self):
        """
        Test that a Certificate object with no data can be written to a data
        stream.
        """
        self._test_write(self.encoding_a, self.certificate_type_a,
                         self.certificate_value_a)

    def test_write_b(self):
        """
        Test that a Certificate object with data can be written to a data
        stream.
        """
        self._test_write(self.encoding_b, self.certificate_type_b,
                         self.certificate_value_b)

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Certificate objects with the same internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)
        b = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_equal_and_empty(self):
        """
        Test that the equality operator returns True when comparing two
        Certificate objects with no internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        Certificate objects with different sets of internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a
        Certificate object with a non-Certificate object.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Certificate objects with the same internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)
        b = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_equal_and_empty(self):
        """
        Test that the inequality operator returns False when comparing two
        Certificate objects with no internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        Certificate objects with different sets of internal data.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        Certificate object with a non-Certificate object.
        """
        a = Certificate(
            certificate_type=self.certificate_type_a,
            certificate_value=self.certificate_value_a)
        b = "invalid"

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that the representation of a Certificate object with data is
        formatted properly.
        """
        certificate = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        certificate_type = "certificate_type={0}".format(
            str(self.certificate_type_b))
        certificate_value = "certificate_value=b'{0}'".format(
            str(self.certificate_value_b))

        expected = "Certificate({0}, {1})".format(
            certificate_type, certificate_value)
        observed = repr(certificate)

        msg = "\nexpected:\n{0}\nobserved:\n{1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)

        # NOTE (peter-hamilton) Testing with eval won't work due to null bytes.

    def test_str(self):
        """
        Test that the string representation of a Certificate object is
        formatted properly.
        """
        certificate = Certificate(
            certificate_type=self.certificate_type_b,
            certificate_value=self.certificate_value_b)

        expected = str(self.certificate_value_b)
        observed = str(certificate)

        msg = "expected:\n{0}\nobserved:\n{1}".format(expected, observed)
        self.assertEqual(expected, observed, msg)



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import six

from kmip.core import enums


def parse_policy(policy):
    result = {}

    for object_type, operation_policies in six.iteritems(policy):
        processed_operation_policies = {}

        for operation, permission in six.iteritems(operation_policies):
            try:
                enum_operation = enums.Operation[operation]
            except Exception:
                raise ValueError(
                    "'{0}' is not a valid Operation value.".format(
                        operation
                    )
                )
            try:
                enum_policy = enums.Policy[permission]
            except Exception:
                raise ValueError(
                    "'{0}' is not a valid Policy value.".format(
                        permission
                    )
                )

            processed_operation_policies[enum_operation] = enum_policy

        try:
            enum_type = enums.ObjectType[object_type]
        except Exception:
            raise ValueError(
                "'{0}' is not a valid ObjectType value.".format(
                    object_type
                )
            )

        result[enum_type] = processed_operation_policies

    return result


def read_policy_from_file(path):
    policy_blob = {}

    with open(path, 'r') as f:
        try:
            policy_blob = json.loads(f.read())
        except Exception as e:
            raise ValueError(
                "Loading the policy file '{}' generated a JSON error: "
                "{}".format(path, e)
            )

    policy_sections = {'groups', 'preset'}
    object_types = set([t.name for t in enums.ObjectType])
    result = {}

    for name, object_policy in policy_blob.items():
        if len(object_policy.keys()) == 0:
            continue

        # Use subset checking to determine what type of policy we have
        sections = set([s for s in six.iterkeys(object_policy)])
        if sections <= policy_sections:
            parsed_policies = dict()

            default_policy = object_policy.get('preset')
            if default_policy:
                parsed_policies['preset'] = parse_policy(default_policy)

            group_policies = object_policy.get('groups')
            if group_policies:
                parsed_group_policies = dict()
                for group_name, group_policy in six.iteritems(group_policies):
                    parsed_group_policies[group_name] = parse_policy(
                        group_policy
                    )
                parsed_policies['groups'] = parsed_group_policies

            result[name] = parsed_policies
        elif sections <= object_types:
            policy = parse_policy(object_policy)
            result[name] = {'preset': policy}
        else:
            invalid_sections = sections - policy_sections - object_types
            raise ValueError(
                "Policy '{}' contains an invalid section named: "
                "{}".format(name, invalid_sections.pop())
            )

    return result


policies = {
    'default': {
        'preset': {
            enums.ObjectType.CERTIFICATE: {
                enums.Operation.LOCATE:             enums.Policy.ALLOW_ALL,
                enums.Operation.CHECK:              enums.Policy.ALLOW_ALL,
                enums.Operation.GET:                enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTES:     enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTE_LIST: enums.Policy.ALLOW_ALL,
                enums.Operation.ADD_ATTRIBUTE:      enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:      enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:       enums.Policy.ALLOW_ALL,
                enums.Operation.ACTIVATE:           enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:            enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:            enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:            enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.SYMMETRIC_KEY: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.PUBLIC_KEY: {
                enums.Operation.LOCATE:             enums.Policy.ALLOW_ALL,
                enums.Operation.CHECK:              enums.Policy.ALLOW_ALL,
                enums.Operation.GET:                enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTES:     enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTE_LIST: enums.Policy.ALLOW_ALL,
                enums.Operation.ADD_ATTRIBUTE:      enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:       enums.Policy.ALLOW_ALL,
                enums.Operation.ACTIVATE:           enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:            enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:            enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:            enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.PRIVATE_KEY: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.SPLIT_KEY: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.TEMPLATE: {
                enums.Operation.LOCATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:     enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST: enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:      enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:   enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:      enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:            enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.SECRET_DATA: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.OPAQUE_DATA: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            },
            enums.ObjectType.PGP_KEY: {
                enums.Operation.REKEY:                enums.Policy.ALLOW_OWNER,
                enums.Operation.REKEY_KEY_PAIR:       enums.Policy.ALLOW_OWNER,
                enums.Operation.DERIVE_KEY:           enums.Policy.ALLOW_OWNER,
                enums.Operation.LOCATE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.CHECK:                enums.Policy.ALLOW_OWNER,
                enums.Operation.GET:                  enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTES:       enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_ATTRIBUTE_LIST:   enums.Policy.ALLOW_OWNER,
                enums.Operation.ADD_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.MODIFY_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.DELETE_ATTRIBUTE:     enums.Policy.ALLOW_OWNER,
                enums.Operation.SET_ATTRIBUTE:        enums.Policy.ALLOW_OWNER,
                enums.Operation.OBTAIN_LEASE:         enums.Policy.ALLOW_OWNER,
                enums.Operation.GET_USAGE_ALLOCATION: enums.Policy.ALLOW_OWNER,
                enums.Operation.ACTIVATE:             enums.Policy.ALLOW_OWNER,
                enums.Operation.REVOKE:               enums.Policy.ALLOW_OWNER,
                enums.Operation.DESTROY:              enums.Policy.ALLOW_OWNER,
                enums.Operation.ARCHIVE:              enums.Policy.ALLOW_OWNER,
                enums.Operation.RECOVER:              enums.Policy.ALLOW_OWNER
            }
        }
    },
    'public': {
        'preset': {
            enums.ObjectType.TEMPLATE: {
                enums.Operation.LOCATE:             enums.Policy.ALLOW_ALL,
                enums.Operation.GET:                enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTES:     enums.Policy.ALLOW_ALL,
                enums.Operation.GET_ATTRIBUTE_LIST: enums.Policy.ALLOW_ALL,
                enums.Operation.ADD_ATTRIBUTE:      enums.Policy.DISALLOW_ALL,
                enums.Operation.MODIFY_ATTRIBUTE:   enums.Policy.DISALLOW_ALL,
                enums.Operation.DELETE_ATTRIBUTE:   enums.Policy.DISALLOW_ALL,
                enums.Operation.SET_ATTRIBUTE:      enums.Policy.DISALLOW_ALL,
                enums.Operation.DESTROY:            enums.Policy.DISALLOW_ALL
            }
        }
    }
}



# Copyright (c) 2018 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests
import six

from kmip.core import exceptions
from kmip.services.server.auth import api
from kmip.services.server.auth import utils


class SLUGSConnector(api.AuthAPI):
    """
    An authentication API connector for a SLUGS service.
    """

    def __init__(self, url=None):
        """
        Construct a SLUGSConnector.

        Args:
            url (string): The base URL for the remote SLUGS instance. Optional,
                defaults to None. Required for authentication.
        """
        self._url = None
        self.users_url = None
        self.groups_url = None

        self.url = url

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        if value is None:
            self._url = None
            self.users_url = None
            self.groups_url = None
        elif isinstance(value, six.string_types):
            self._url = value
            if not self._url.endswith("/"):
                self._url += "/"
            self.users_url = self._url + "users/{}"
            self.groups_url = self.users_url + "/groups"
        else:
            raise TypeError("URL must be a string.")

    def authenticate(self,
                     connection_certificate=None,
                     connection_info=None,
                     request_credentials=None):
        """
        Query the configured SLUGS service with the provided credentials.

        Args:
            connection_certificate (cryptography.x509.Certificate): An X.509
                certificate object obtained from the connection being
                authenticated. Required for SLUGS authentication.
            connection_info (tuple): A tuple of information pertaining to the
                connection being authenticated, including the source IP address
                and a timestamp (e.g., ('127.0.0.1', 1519759267.467451)).
                Optional, defaults to None. Ignored for SLUGS authentication.
            request_credentials (list): A list of KMIP Credential structures
                containing credential information to use for authentication.
                Optional, defaults to None. Ignored for SLUGS authentication.
        """
        if (self.users_url is None) or (self.groups_url is None):
            raise exceptions.ConfigurationError(
                "The SLUGS URL must be specified."
            )

        user_id = utils.get_client_identity_from_certificate(
            connection_certificate
        )

        try:
            response = requests.get(self.users_url.format(user_id))
        except Exception:
            raise exceptions.ConfigurationError(
                "A connection could not be established using the SLUGS URL."
            )
        if response.status_code == 404:
            raise exceptions.PermissionDenied(
                "Unrecognized user ID: {}".format(user_id)
            )

        response = requests.get(self.groups_url.format(user_id))
        if response.status_code == 404:
            raise exceptions.PermissionDenied(
                "Group information could not be retrieved for user ID: "
                "{}".format(user_id)
            )

        return user_id, response.json().get('groups')



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import primitives
from kmip.core import utils


class TestDateTime(testtools.TestCase):
    """
    Test suite for the DateTime primitive.

    Since DateTime is a subclass of LongInteger, the bulk of the functionality
    tests are omitted due to redundancy with the LongInteger test suite.
    """

    def setUp(self):
        super(TestDateTime, self).setUp()

        self.value = 1335514341
        self.encoding = (
            b'\x42\x00\x00\x09\x00\x00\x00\x08\x00\x00\x00\x00\x4F\x9A\x54'
            b'\xE5')

    def tearDown(self):
        super(TestDateTime, self).tearDown()

    def test_init(self):
        """
        Test that a DateTime can be instantiated.
        """
        date_time = primitives.DateTime(1)
        self.assertEqual(1, date_time.value)

    def test_init_unset(self):
        """
        Test that a DateTime can be instantiated with no input.
        """
        date_time = primitives.DateTime()
        self.assertNotEqual(date_time.value, None)

    def test_read(self):
        """
        Test that a DateTime can be read from a byte stream.
        """
        stream = utils.BytearrayStream(self.encoding)
        date_time = primitives.DateTime()
        date_time.read(stream)
        self.assertEqual(self.value, date_time.value)

    def test_write(self):
        """
        Test that a DateTime can be written to a byte stream.
        """
        stream = utils.BytearrayStream()
        date_time = primitives.DateTime(self.value)
        date_time.write(stream)

        result = stream.read()
        self.assertEqual(len(self.encoding), len(result))
        self.assertEqual(self.encoding, result)

    def test_repr(self):
        """
        Test that the representation of a DateTime is formatted properly.
        """
        date_time = primitives.DateTime(1439299135)
        value = "value={0}".format(date_time.value)
        tag = "tag={0}".format(date_time.tag)
        r = "DateTime({0}, {1})".format(value, tag)

        self.assertEqual(r, repr(date_time))

    def test_str(self):
        """
        Test that the string representation of a DateTime is formatted
        properly.
        """
        expected = 'Tue Aug 11 13:18:55 2015'
        date_time = primitives.DateTime(1439299135)

        self.assertEqual(expected, str(date_time))



# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from kmip.core import attributes
from kmip.core import enums
from kmip.core import primitives
from kmip.core import utils
from kmip.core.messages.payloads import base


class SignatureVerifyRequestPayload(base.RequestPayload):
    """
    A request payload for the SignatureVerify operation.

    Attributes:
        unique_identifier: The unique ID of the key to use for signature
            verification.
        cryptographic_parameters: A collection of settings relevant for the
            signature verification process.
        data: The data that was signed.
        digested_data: The digested data to be verified.
        signature_data: The signature to be verified.
        correlation_value: An identifier for an existing, incomplete operation
            this payload hooks into.
        init_indicator: A boolean indicating whether or not the payload is the
            first in a series for a multi-payload operation.
        final_indicator: A boolean indicating whether or not the payload is
            the last in a series for a multi-payload operation.
    """

    def __init__(self,
                 unique_identifier=None,
                 cryptographic_parameters=None,
                 data=None,
                 digested_data=None,
                 signature_data=None,
                 correlation_value=None,
                 init_indicator=None,
                 final_indicator=None):
        """
        Construct a SignatureVerify request payload struct.

        Args:
            unique_identifier (string): The ID of the managed object (e.g.,
                a public key) to be used for signature verification. Optional,
                defaults to None.
            cryptographic_parameters (CryptographicParameters): A
                CryptographicParameters struct containing the settings for
                the signature verification operation. Optional, defaults to
                None.
            data (bytes): The bytes representing the original data that was
                signed. Optional, defaults to None.
            digested_data (bytes): The bytes representing the digested data to
                be verified with the signature. Optional, defaults to None.
            signature_data (bytes): The bytes representing the signature to be
                verified. Optional, defaults to None.
            correlation_value (bytes): The bytes representing a correlation
                value, allowing the linking together of individual payloads
                for a single overarching operation. Optional, defaults to None.
            init_indicator (boolean): A boolean value indicating whether or not
                the payload is the first in a series for a multi-payload
                operation. Optional, defaults to None.
            final_indicator (boolean): A boolean value indicating whether or
                not the payload is the last in a series for a multi-payload
                operation. Optional, defaults to None.
        """
        super(SignatureVerifyRequestPayload, self).__init__()

        self._unique_identifier = None
        self._cryptographic_parameters = None
        self._data = None
        self._digested_data = None
        self._signature_data = None
        self._correlation_value = None
        self._init_indicator = None
        self._final_indicator = None

        self.unique_identifier = unique_identifier
        self.cryptographic_parameters = cryptographic_parameters
        self.data = data
        self.digested_data = digested_data
        self.signature_data = signature_data
        self.correlation_value = correlation_value
        self.init_indicator = init_indicator
        self.final_indicator = final_indicator

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    @property
    def cryptographic_parameters(self):
        return self._cryptographic_parameters

    @cryptographic_parameters.setter
    def cryptographic_parameters(self, value):
        if value is None:
            self._cryptographic_parameters = None
        elif isinstance(value, attributes.CryptographicParameters):
            self._cryptographic_parameters = value
        else:
            raise TypeError(
                "Cryptographic parameters must be a CryptographicParameters "
                "struct."
            )

    @property
    def data(self):
        if self._data:
            return self._data.value
        else:
            return None

    @data.setter
    def data(self, value):
        if value is None:
            self._data = None
        elif isinstance(value, six.binary_type):
            self._data = primitives.ByteString(
                value=value,
                tag=enums.Tags.DATA
            )
        else:
            raise TypeError("Data must be bytes.")

    @property
    def digested_data(self):
        if self._digested_data:
            return self._digested_data.value
        else:
            return None

    @digested_data.setter
    def digested_data(self, value):
        if value is None:
            self._digested_data = None
        elif isinstance(value, six.binary_type):
            self._digested_data = primitives.ByteString(
                value=value,
                tag=enums.Tags.DIGESTED_DATA
            )
        else:
            raise TypeError("Digested data must be bytes.")

    @property
    def signature_data(self):
        if self._signature_data:
            return self._signature_data.value
        else:
            return None

    @signature_data.setter
    def signature_data(self, value):
        if value is None:
            self._signature_data = None
        elif isinstance(value, six.binary_type):
            self._signature_data = primitives.ByteString(
                value=value,
                tag=enums.Tags.SIGNATURE_DATA
            )
        else:
            raise TypeError("Signature data must be bytes.")

    @property
    def correlation_value(self):
        if self._correlation_value:
            return self._correlation_value.value
        else:
            return None

    @correlation_value.setter
    def correlation_value(self, value):
        if value is None:
            self._correlation_value = None
        elif isinstance(value, six.binary_type):
            self._correlation_value = primitives.ByteString(
                value=value,
                tag=enums.Tags.CORRELATION_VALUE
            )
        else:
            raise TypeError("Correlation value must be bytes.")

    @property
    def init_indicator(self):
        if self._init_indicator:
            return self._init_indicator.value
        else:
            return None

    @init_indicator.setter
    def init_indicator(self, value):
        if value is None:
            self._init_indicator = None
        elif isinstance(value, bool):
            self._init_indicator = primitives.Boolean(
                value=value,
                tag=enums.Tags.INIT_INDICATOR
            )
        else:
            raise TypeError("Init indicator must be a boolean.")

    @property
    def final_indicator(self):
        if self._final_indicator:
            return self._final_indicator.value
        else:
            return None

    @final_indicator.setter
    def final_indicator(self, value):
        if value is None:
            self._final_indicator = None
        elif isinstance(value, bool):
            self._final_indicator = primitives.Boolean(
                value=value,
                tag=enums.Tags.FINAL_INDICATOR
            )
        else:
            raise TypeError("Final indicator must be a boolean.")

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the SignatureVerify request payload and decode
        it into its constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the data attribute is missing from the
                encoded payload.
        """
        super(SignatureVerifyRequestPayload, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_stream):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_stream,
                kmip_version=kmip_version
            )
        if self.is_tag_next(enums.Tags.CRYPTOGRAPHIC_PARAMETERS, local_stream):
            self._cryptographic_parameters = \
                attributes.CryptographicParameters()
            self._cryptographic_parameters.read(
                local_stream,
                kmip_version=kmip_version
            )
        if self.is_tag_next(enums.Tags.DATA, local_stream):
            self._data = primitives.ByteString(tag=enums.Tags.DATA)
            self._data.read(local_stream, kmip_version=kmip_version)
        if self.is_tag_next(enums.Tags.DIGESTED_DATA, local_stream):
            self._digested_data = primitives.ByteString(
                tag=enums.Tags.DIGESTED_DATA
            )
            self._digested_data.read(local_stream, kmip_version=kmip_version)
        if self.is_tag_next(enums.Tags.SIGNATURE_DATA, local_stream):
            self._signature_data = primitives.ByteString(
                tag=enums.Tags.SIGNATURE_DATA
            )
            self._signature_data.read(local_stream, kmip_version=kmip_version)
        if self.is_tag_next(enums.Tags.CORRELATION_VALUE, local_stream):
            self._correlation_value = primitives.ByteString(
                tag=enums.Tags.CORRELATION_VALUE
            )
            self._correlation_value.read(
                local_stream,
                kmip_version=kmip_version
            )
        if self.is_tag_next(enums.Tags.INIT_INDICATOR, local_stream):
            self._init_indicator = primitives.Boolean(
                tag=enums.Tags.INIT_INDICATOR
            )
            self._init_indicator.read(local_stream, kmip_version=kmip_version)
        if self.is_tag_next(enums.Tags.FINAL_INDICATOR, local_stream):
            self._final_indicator = primitives.Boolean(
                tag=enums.Tags.FINAL_INDICATOR
            )
            self._final_indicator.read(local_stream, kmip_version=kmip_version)

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the SignatureVerify request payload to a
        stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the data attribute is not defined.
        """
        local_stream = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._cryptographic_parameters:
            self._cryptographic_parameters.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._data:
            self._data.write(local_stream, kmip_version=kmip_version)
        if self._digested_data:
            self._digested_data.write(local_stream, kmip_version=kmip_version)
        if self._signature_data:
            self._signature_data.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._correlation_value:
            self._correlation_value.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._init_indicator:
            self._init_indicator.write(
                local_stream,
                kmip_version=kmip_version
            )
        if self._final_indicator:
            self._final_indicator.write(
                local_stream,
                kmip_version=kmip_version
            )

        self.length = local_stream.length()
        super(SignatureVerifyRequestPayload, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, SignatureVerifyRequestPayload):
            if self.unique_identifier != other.unique_identifier:
                return False
            elif self.cryptographic_parameters != \
                    other.cryptographic_parameters:
                return False
            elif self.data != other.data:
                return False
            elif self.digested_data != other.digested_data:
                return False
            elif self.signature_data != other.signature_data:
                return False
            elif self.correlation_value != other.correlation_value:
                return False
            elif self.init_indicator != other.init_indicator:
                return False
            elif self.final_indicator != other.final_indicator:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, SignatureVerifyRequestPayload):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "unique_identifier='{0}'".format(self.unique_identifier),
            "cryptographic_parameters={0}".format(
                repr(self.cryptographic_parameters)
            ),
            "data={0}".format(self.data),
            "digested_data={0}".format(self.digested_data),
            "signature_data={0}".format(self.signature_data),
            "correlation_value={0}".format(self.correlation_value),
            "init_indicator={0}".format(self.init_indicator),
            "final_indicator={0}".format(self.final_indicator)
        ])
        return "SignatureVerifyRequestPayload({0})".format(args)

    def __str__(self):
        return str({
            'unique_identifier': self.unique_identifier,
            'cryptographic_parameters': self.cryptographic_parameters,
            'data': self.data,
            'digested_data': self.digested_data,
            'signature_data': self.signature_data,
            'correlation_value': self.correlation_value,
            'init_indicator': self.init_indicator,
            'final_indicator': self.final_indicator
        })


class SignatureVerifyResponsePayload(base.ResponsePayload):
    """
    A response payload for the SignatureVerify operation.

    Attributes:
        unique_identifier: The unique ID of the key used for signature
            verification.
        validity_indicator: The validity of the verified signature.
        data: Recovered data produced by the signature verification process.
        correlation_value: An identifier for an existing, incomplete operation
            this payload is a part of.
    """

    def __init__(self,
                 unique_identifier=None,
                 validity_indicator=None,
                 data=None,
                 correlation_value=None):
        """
        Construct a SignatureVerify response payload struct.

        Args:
            unique_identifier (string): The ID of the managed object (e.g.,
                a public key) used for signature verification. Optional,
                defaults to None. Required for read/write.
            validity_indicator (ValidityIndicator): A ValidityIndicator
                enumeration that specifies the validity of the signature.
                Optional, defaults to None. Required for read/write.
            data (bytes): The bytes representing any data recovered during the
                signature verification process. Optional, defaults to None.
            correlation_value (bytes): The bytes representing a correlation
                value, allowing the linking together of individual payloads
                for a single overarching operation. Optional, defaults to None.
        """
        super(SignatureVerifyResponsePayload, self).__init__()

        self._unique_identifier = None
        self._validity_indicator = None
        self._data = None
        self._correlation_value = None

        self.unique_identifier = unique_identifier
        self.validity_indicator = validity_indicator
        self.data = data
        self.correlation_value = correlation_value

    @property
    def unique_identifier(self):
        if self._unique_identifier:
            return self._unique_identifier.value
        else:
            return None

    @unique_identifier.setter
    def unique_identifier(self, value):
        if value is None:
            self._unique_identifier = None
        elif isinstance(value, six.string_types):
            self._unique_identifier = primitives.TextString(
                value=value,
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
        else:
            raise TypeError("Unique identifier must be a string.")

    @property
    def validity_indicator(self):
        if self._validity_indicator:
            return self._validity_indicator.value
        else:
            return None

    @validity_indicator.setter
    def validity_indicator(self, value):
        if value is None:
            self._validity_indicator = None
        elif isinstance(value, enums.ValidityIndicator):
            self._validity_indicator = primitives.Enumeration(
                enums.ValidityIndicator,
                value=value,
                tag=enums.Tags.VALIDITY_INDICATOR
            )
        else:
            raise TypeError(
                "Validity indicator must be a ValidityIndicator enumeration."
            )

    @property
    def data(self):
        if self._data:
            return self._data.value
        else:
            return None

    @data.setter
    def data(self, value):
        if value is None:
            self._data = None
        elif isinstance(value, six.binary_type):
            self._data = primitives.ByteString(
                value=value,
                tag=enums.Tags.DATA
            )
        else:
            raise TypeError("Data must be bytes.")

    @property
    def correlation_value(self):
        if self._correlation_value:
            return self._correlation_value.value
        else:
            return None

    @correlation_value.setter
    def correlation_value(self, value):
        if value is None:
            self._correlation_value = None
        elif isinstance(value, six.binary_type):
            self._correlation_value = primitives.ByteString(
                value=value,
                tag=enums.Tags.CORRELATION_VALUE
            )
        else:
            raise TypeError("Correlation value must be bytes.")

    def read(self, input_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the SignatureVerify response payload and decode
        it into its constituent parts.

        Args:
            input_stream (stream): A data stream containing encoded object
                data, supporting a read method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the data attribute is missing from the
                encoded payload.
        """
        super(SignatureVerifyResponsePayload, self).read(
            input_stream,
            kmip_version=kmip_version
        )
        local_stream = utils.BytearrayStream(input_stream.read(self.length))

        if self.is_tag_next(enums.Tags.UNIQUE_IDENTIFIER, local_stream):
            self._unique_identifier = primitives.TextString(
                tag=enums.Tags.UNIQUE_IDENTIFIER
            )
            self._unique_identifier.read(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Parsed payload encoding is missing the unique identifier "
                "field."
            )
        if self.is_tag_next(enums.Tags.VALIDITY_INDICATOR, local_stream):
            self._validity_indicator = primitives.Enumeration(
                enums.ValidityIndicator,
                tag=enums.Tags.VALIDITY_INDICATOR
            )
            self._validity_indicator.read(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Parsed payload encoding is missing the validity indicator "
                "field."
            )
        if self.is_tag_next(enums.Tags.DATA, local_stream):
            self._data = primitives.ByteString(tag=enums.Tags.DATA)
            self._data.read(local_stream, kmip_version=kmip_version)
        if self.is_tag_next(enums.Tags.CORRELATION_VALUE, local_stream):
            self._correlation_value = primitives.ByteString(
                tag=enums.Tags.CORRELATION_VALUE
            )
            self._correlation_value.read(
                local_stream,
                kmip_version=kmip_version
            )

        self.is_oversized(local_stream)

    def write(self, output_stream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the SignatureVerify response payload to a
        stream.

        Args:
            output_stream (stream): A data stream in which to encode object
                data, supporting a write method; usually a BytearrayStream
                object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.

        Raises:
            ValueError: Raised if the data attribute is not defined.
        """
        local_stream = utils.BytearrayStream()

        if self._unique_identifier:
            self._unique_identifier.write(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Payload is missing the unique identifier field."
            )
        if self._validity_indicator:
            self._validity_indicator.write(
                local_stream,
                kmip_version=kmip_version
            )
        else:
            raise ValueError(
                "Payload is missing the validity indicator field."
            )
        if self._data:
            self._data.write(local_stream, kmip_version=kmip_version)
        if self._correlation_value:
            self._correlation_value.write(
                local_stream,
                kmip_version=kmip_version
            )

        self.length = local_stream.length()
        super(SignatureVerifyResponsePayload, self).write(
            output_stream,
            kmip_version=kmip_version
        )
        output_stream.write(local_stream.buffer)

    def __eq__(self, other):
        if isinstance(other, SignatureVerifyResponsePayload):
            if self.unique_identifier != other.unique_identifier:
                return False
            elif self.validity_indicator != other.validity_indicator:
                return False
            elif self.data != other.data:
                return False
            elif self.correlation_value != other.correlation_value:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, SignatureVerifyResponsePayload):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        args = ", ".join([
            "unique_identifier='{0}'".format(self.unique_identifier),
            "validity_indicator={0}".format(self.validity_indicator),
            "data={0}".format(self.data),
            "correlation_value={0}".format(self.correlation_value)
        ])
        return "SignatureVerifyResponsePayload({0})".format(args)

    def __str__(self):
        return str({
            'unique_identifier': self.unique_identifier,
            'validity_indicator': self.validity_indicator,
            'data': self.data,
            'correlation_value': self.correlation_value
        })
